-- $Name:Ныряльщик$
-- $Version:1.03$
-- $Author:Luka$
-- $Info: Ныряльщики. Так называют тех, кто погружается каждую ночь в осознанный сон, чтобы вернуться из него с жемчугом... или же с безумием. Вас ждёт сюрреалистичный квест в духе серии Myst на основе "классического" Instead.$

--    Ныряльщик/The Diver
--    Copyright (C) 2024  Luka (ot-luki@yandex.ru)
--
--    This program is free software: you can redistribute it and/or modify
--    it under the terms of the GNU General Public License as published by
--    the Free Software Foundation, either version 3 of the License, or
--    (at your option) any later version.
--
--    This program is distributed in the hope that it will be useful,
--    but WITHOUT ANY WARRANTY; without even the implied warranty of
--    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--    GNU General Public License for more details.
--
--    You should have received a copy of the GNU General Public License
--    along with this program.  If not, see <https://www.gnu.org/licenses/>.


require "fmt" -- некоторые функции форматирования
require "noinv"
require "theme"
require "autotheme"
require 'fading' -- затенение

fmt.para = true -- включить режим параграфов (отступы)

game.act = 'Действие на предмет не прописано.';
game.use = 'Это не имеет смысла.';
game.inv = 'Описание предмета не задано.';

xact.walk = walk;
-- Переход с затенением. Отключать затенение приходится на выходе из локации
xact.fwalk = function (rm)
    fading.change {'fadeblack', max = 64, delay = 20}
    walk(rm)
end

xact.changeAlign = function()
    --dprint(theme.get("win.align"))
    if myalign=='' then myalign=theme.get "win.align" end
    if myalign == 'justify' then myalign = 'left' else myalign = 'justify' end
    theme.set("win.align", myalign)
end

function init()
    std.strip_call = false
    fading.change{'none'}
    take 'Веретено'
end

function start()
    if myalign~='' then theme.set("win.align", myalign) end  --; justify/left
end

global {
bpearl = 0;       -- Счетчик собранных жемчужин-секретов
picext = '.png';  -- Расширение для изображений (кроме прописанных в конфиге темы)
myalign = '';
}





----------- Изменения порядка отображения сообщений ---------------------
function std.world:display(state)
        local l, av, pv
        local reaction = self:reaction() or nil
        if state then
            reaction = iface:em(reaction)
            av, pv = self:events()
            av = iface:em(av)
            pv = iface:em(pv)
            l = self.player:look() -- objects [and scene]
        end
        l = std.par( std.scene_delim,
                     reaction or false,
                     av or false,
                     pv or false,
                     l or false
                     ) or ''
        return l
end;


--------------------- Система работы с набором строк --------------------

declare 'mystr' {}
include 'strings.lua'

-- Переносы строк через ^
for key, val in pairs(mystr) do
   mystr[key] = (val:gsub("\n", "^")):gsub("%-%-%-", "—")
end

function str(s)
    --dprint(s)
    local st, key
    if s.nam==nil   then key = s
                    else key = s.nam
                    end
    st = mystr[key]
    if st==nil   then st='^Ошибка! Отсутствует строка с ключом <'..key..'>^'
                 end
    --dprint(st)
    return st
end

function pa(s)
    pn(str(s))
end

include 'obj_1p.lua'

------------------------ Перенос слов ----------------------------
obj {
nam = '$w';
act = function(s, str)
      return ('{$fmt nb|'..string.gsub(str,'`','}{$fmt nb|')..'}')
      end;
} 
-- использование: внутри текста пишем {$w|тут`какоето`очень`длинное`слово`которое`надо`разделить}
------------------------ Главное меню ----------------------------

room {
    nam = 'main';
    disp = 'Главное меню';
    noinv = true;
    pic = 'img/logo'..picext;
    decor = fmt.r([[
    "Сон — это скрытая ^
    маленькая дверь, ^
    ведущая в самые ^
    потаённые и сокро-^
    венные уголки души ^
    и открывающаяся в ^
    космическую ночь."^^
    Карл Густав Юнг]])..fmt.b([[
^^^^^^^^
- {@ fwalk Вступление|Играть}^
- {@ walk От_автора|Об игре}^
- {@ walk Помощь|Помощь}
]]);
}


room{
    nam = 'От_автора';
    disp = 'Об игре';
    noinv = true;
    pic = 'img/logo'..picext;
    way = {'main', 'Помощь'};
    decor = [[Автор идеи, программирование, текст, иллюстрации, оформление темы: Лука^ 
    Движок (INSTEAD): Пётр Косых^^
    Исходный код игры распространяется по лицензии GNU GPL v3.^^^^^
    Пишите письма: ^
    ot-luki@yandex.ru
    ]];
    }

room{
    nam = 'Помощь';
    noinv = true;
    pic = 'img/logo'..picext;
    way = {'main', 'От_автора'};
    decor = [[
    - Нажмите {@ changeAlign|здесь}, если хотите изменить выравнивание текста в основном окне (по левому краю или по ширине).^
    - В игре не предусмотрено летальных исходов или непроходимых ситуаций.^
    - Вы начинаете игру с веретеном — это ваш талисман. Нажмите на него дважды чтобы оставить в текущей локации и вы сможете вернуться сюда из любого места. Но это не единственное его предназначение.^
    - Кувшин, который вы скоро найдёте, тоже активируется двойным нажатием. Обратите внимание, что персонаж по-разному его использует в зависимости от того, есть ли что внутри.^
    - Некоторые ссылки пропадают после первого нажатия на них. Они не используются для решения загадок и добавлены ради антуража и раскрытия вселенной игры.^
    - В игре вы можете найти чёрные жемчужины — это "секреты", они не отмечаются в инвентаре и не используются в решении загадок.^
    - Если вам нужна помощь по использованию интерфейса Instead, следует ознакомиться с поставляемым вместе с интерпретатором туториалом: Меню — Выбор игры — Обучение.^
    - Игра имеет отдельную тему для мобильных телефонов: Меню — Выбор темы — Ныряльщик. Мобильная.^
    - Краткое прохождение игры вы можете найти в файле solution.txt.
    ]];
    }


obj {
nam = 'Веретено';
act = function(s)
        replace('Крутиться...','Веретено')
        remove('Веретено', here())
        return 'Вы подбираете веретено'
        end;
dsc = '^Здесь лежит {веретено}.';
neverused = true;
mydsc = 'Отливающий серебром {$w|ве`ре`те`но`об`раз`ный} предмет. Длиной примерно с вашу ладонь. На ощупь металлический и довольно тяжёлый. Это ваш амулет. Оставив его где-нибудь вы можете вернуться в это место, закрутившись волчком. А ещё он твёрдый и довольно острый с конца. (Кликните на него дважды в инвентаре, если хотите оставить его в текущей локации)';
inv = function(s)
        if s.neverused then
            s.neverused = false
            return s.mydsc
            end
        replace('Веретено', 'Крутиться...')
        place('Веретено')
        return 'Вы оставляете здесь веретено.'
        end;
used = function(s,w)
            if w^'Цепь с крюком' then 
                pn 'У вас нет идей, как их соединить.'
            elseif string.match(w.nam,'Вес%d') then
                pn 'Веретено будет полегче.'
            elseif w^'Кувшин' and _'Кувшин'.sound~='' then -- Со звуком
                pn 'Если быть неосторожным, крышка приоткроется и звук выберется наружу.'
            elseif w^'Кувшин' and not _'Кувшин'.withwater then -- Без воды без звука
                pn 'Может попробовать наоборот?'
            elseif w^'Кувшин' and _'Кувшин'.withwater then -- С водой без звука
                pn 'Можно было бы ополоснуть веретено, но не похоже, чтобы оно в этом нуждалось.'
            elseif w^'Веретено' then
                pn 'Ошибка!'
            elseif w^'Молот' then
                pn 'Зачем? Да и наковальни тут, кажется, нигде нет.'
            elseif w^'Клещи' then
                pn 'Веретено не разбирается.'
            elseif w^'Гвоздь' then
                pn 'Вы проводите гвоздём по веретену, но на нём ни остаётся даже царапины.'
            elseif w^'Доски' then
                pn 'Вам не кажется хорошей идеей расщеплять доску на части.'
            elseif w^'Крест' then
                pn 'Кресту это ничем не поможет.'
            elseif w^'Слеза' then
                pn 'Не соединяется.'
            else
                return false
            end
        end;
}

obj {
nam = 'Крутиться...';
inv = function(s)
        if where(_'Веретено')==here() then return 'Вы раскручиваетесь вокруг собственной оси. У вас слегка кружится голова, но сон остаётся прежним.' end
        pn 'Вы раскручиваетесь вокруг собственной оси, быстрее и быстрее, пока сон вокруг вас не начинает меняться.'
        walk(where(_'Веретено'))
        end;
use = function(s,w)
        pn 'Это не предмет, вы не можете его ни на что применить. Выберите пункт дважды, если хотите вернуться к месту, где вы оставили веретено.'
        end;
used = function(s,w)
        pn 'На это невозможно применять предметы.'
        end;
}

include 'intro.lua'
include 'outside.lua'
include 'jug.lua'
include 'face.lua'
include 'west.lua'
include 'hall.lua'
include 'south.lua'
include 'north.lua'
include 'east.lua'
include 'final.lua'
include 'weight.lua'
include 'bpearl.lua'
